@@ -21,30 +21,49 @@ def get_info():
     info = dict()
 
     result = subprocess.run(
-        [sys.executable, 'setup.py', '--name'],
+        [sys.executable, 'setup.py', '--name', '--description'],
         stdout=subprocess.PIPE, check=True, cwd=module_dir)
-    info['name'] = result.stdout.decode('utf-8').strip()
+    info['name'], info['description'] = (
+        result.stdout.decode('utf-8').strip().splitlines())
 
     result = subprocess.run(
         [sys.executable, 'setup.py', '--version'],
         stdout=subprocess.PIPE, check=True, cwd=module_dir)
     version = result.stdout.decode('utf-8').strip()
-    if 'dev' in version:
+    major_version, minor_version, _ = version.split('.', 2)
+    major_version = int(major_version)
+    minor_version = int(minor_version)
+    if minor_version % 2:
         info['series'] = 'latest'
+        info['branch'] = 'branch/default'
     else:
         info['series'] = '.'.join(version.split('.', 2)[:2])
+        info['branch'] = 'branch/' + info['series']
 
     return info
 
 
 info = get_info()
 
+html_theme = 'sphinx_book_theme'
+html_theme_options = {
+    'repository_provider': 'gitlab',
+    'repository_url': 'https://code.tryton.org/tryton',
+    'repository_branch': info['branch'],
+    'use_source_button': True,
+    'use_edit_page_button': True,
+    'use_repository_button': True,
+    'use_download_button': False,
+    'path_to_docs': 'tryton/doc',
+    }
+html_title = info['description']
 master_doc = 'index'
 project = info['name']
 release = version = info['series']
 default_role = 'ref'
 highlight_language = 'none'
 extensions = [
+    'sphinx_copybutton',
     'sphinx.ext.intersphinx',
     ]
 intersphinx_mapping = {
