@@ -81,9 +81,10 @@ def populate(menu, model, record, title='', field=None, context=None):
             template=template)
 
     def edit(menuitem):
+        view_ids = field.attrs.get('view_ids', '').split(',')
         with Window(hide_current=True, allow_similar=True):
             Window.create(model,
-                view_ids=field.attrs.get('view_ids', '').split(','),
+                view_ids=view_ids[1:],
                 res_id=id_(record),
                 mode=['form'],
                 name=field.attrs.get('string'),
