@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import datetime
 
 
@@ -9,7 +9,7 @@ class DatesPeriod():
     """
 
     def __init__(self, start, end):
-        assert type(start) == type(end)
+        assert type(start) is type(end)
         self.start = start
         self.end = end
 
