@@ -1,41 +1,44 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import operator
 import gettext
+import operator
 
-from gi.repository import Gtk
+from gi.repository import GLib, Gtk
+
+from tryton.common import IconFactory, Tooltips, get_align, node_attributes
+from tryton.common.button import Button
+from tryton.common.focus import (
+    find_first_focus_widget, find_focusable_child, find_focused_child,
+    get_invisible_ancestor, next_focus_widget)
+from tryton.common.underline import set_underline
+from tryton.config import CONFIG
+from tryton.gui.window.code_scanner import CodeScanner
 
 from . import View, XMLViewParser
-from tryton.common.focus import (get_invisible_ancestor, find_focused_child,
-    next_focus_widget, find_focusable_child, find_first_focus_widget)
-from tryton.common import Tooltips, node_attributes, IconFactory, get_align
-from tryton.common.underline import set_underline
-from tryton.common.button import Button
-from tryton.config import CONFIG
-from .form_gtk.calendar_ import Date, Time, DateTime
+from .form_gtk.binary import Binary
+from .form_gtk.calendar_ import Date, DateTime, Time
+from .form_gtk.char import Char, Password
+from .form_gtk.checkbox import CheckBox
+from .form_gtk.dictionary import DictWidget
 from .form_gtk.document import Document
 from .form_gtk.float import Float
+from .form_gtk.image import Image as Image2
 from .form_gtk.integer import Integer
-from .form_gtk.selection import Selection
-from .form_gtk.char import Char, Password
-from .form_gtk.timedelta import TimeDelta
-from .form_gtk.checkbox import CheckBox
-from .form_gtk.reference import Reference
-from .form_gtk.binary import Binary
-from .form_gtk.textbox import TextBox
-from .form_gtk.one2many import One2Many
 from .form_gtk.many2many import Many2Many
 from .form_gtk.many2one import Many2One
-from .form_gtk.url import Email, URL, CallTo, SIP, HTML
-from .form_gtk.image import Image as Image2
-from .form_gtk.progressbar import ProgressBar
-from .form_gtk.one2one import One2One
-from .form_gtk.richtextbox import RichTextBox
-from .form_gtk.dictionary import DictWidget
 from .form_gtk.multiselection import MultiSelection
+from .form_gtk.one2many import One2Many
+from .form_gtk.one2one import One2One
+from .form_gtk.progressbar import ProgressBar
 from .form_gtk.pyson import PYSON
-from .form_gtk.state_widget import (Label, VBox, Image, Frame, ScrolledWindow,
-    Notebook, Expander, Link)
+from .form_gtk.reference import Reference
+from .form_gtk.richtextbox import RichTextBox
+from .form_gtk.selection import Selection
+from .form_gtk.state_widget import (
+    Expander, Frame, Image, Label, Link, Notebook, ScrolledWindow, VBox)
+from .form_gtk.textbox import TextBox
+from .form_gtk.timedelta import TimeDelta
+from .form_gtk.url import HTML, SIP, URL, CallTo, Email
 
 _ = gettext.gettext
 
@@ -160,7 +163,6 @@ class HContainer(_Container):
 class FormXMLViewParser(XMLViewParser):
 
     WIDGETS = {
-        'biginteger': Integer,
         'binary': Binary,
         'boolean': CheckBox,
         'callto': CallTo,
@@ -277,13 +279,18 @@ class FormXMLViewParser(XMLViewParser):
             return
         vbox = VBox(attrs=attributes)
         if attributes.get('string'):
+            attributes.setdefault('xexpand', 0)
+            attributes.setdefault('xalign', 0)
+            attributes.setdefault('yalign', 0.5)
             label = Label(label=attributes['string'], attrs=attributes)
             label.set_halign(get_align(
-                    attributes.get('xalign', 0.0),
-                    bool(attributes.get('xexpand', True))))
+                    attributes['xalign'],
+                    bool(attributes.get('xexpand'))))
             label.set_valign(get_align(
-                    attributes.get('yalign', 0.5),
-                    bool(attributes.get('yexpand', False))))
+                    attributes['yalign'],
+                    bool(attributes.get('yexpand'))))
+            label.props.xalign = float(attributes['xalign'])
+            label.props.yalign = float(attributes['yalign'])
             vbox.pack_start(label, expand=True, fill=True, padding=0)
             self.view.state_widgets.append(label)
             if name:
@@ -301,13 +308,17 @@ class FormXMLViewParser(XMLViewParser):
             attributes['xalign'] = 0.0
 
         attributes.setdefault('xexpand', 0)
+        attributes.setdefault('xalign', 1)
+        attributes.setdefault('yalign', 0.5)
         label = Label(label=attributes.get('string', ''), attrs=attributes)
         label.set_halign(get_align(
-                attributes.get('xalign', 1.0),
+                attributes['xalign'],
                 bool(attributes.get('xexpand'))))
         label.set_valign(get_align(
                 attributes.get('yalign', 0.5),
                 bool(attributes.get('yexpand'))))
+        label.props.xalign = float(attributes['xalign'])
+        label.props.yalign = float(attributes['yalign'])
         label.set_angle(int(attributes.get('angle', 0)))
         self.view.state_widgets.append(label)
         self.container.add(label, attributes)
@@ -379,7 +390,6 @@ class FormXMLViewParser(XMLViewParser):
         group = Container.constructor(
             int(attributes.get('col', 4)),
             attributes.get('homogeneous', False))
-        self.parse_child(node, group)
 
         if 'name' in attributes and attributes['name'] == self.exclude_field:
             self.container.add(None, attributes)
@@ -403,6 +413,9 @@ class FormXMLViewParser(XMLViewParser):
                 bool(attributes.get('yexpand'))))
         self.view.state_widgets.append(widget)
         self.container.add(widget, attributes)
+        # Parse the children at the end to preserve the order of the state
+        # widgets
+        self.parse_child(node, group)
 
     def _parse_hpaned(self, node, attributes):
         self._parse_paned(node, attributes, Gtk.HPaned)
@@ -459,6 +472,25 @@ class ViewForm(View):
 
         super().__init__(view_id, screen, xml)
 
+        self.creatable = bool(int(self.attributes.get('creatable', 1)))
+
+        if self.attributes.get('scan_code'):
+            code_scanner_btn = Button({
+                    'string': _("Scan"),
+                    'icon': 'tryton-barcode-scanner',
+                    'states': self.attributes.get('scan_code_states', {}),
+                    })
+            code_scanner_btn.set_image(IconFactory.get_image(
+                    'tryton-barcode-scanner', Gtk.IconSize.BUTTON))
+            code_scanner_btn.set_always_show_image(True)
+            code_scanner_btn.set_receives_default(False)
+            vbox.pack_start(
+                code_scanner_btn, expand=False, fill=True, padding=2)
+            code_scanner_btn.connect(
+                'clicked', lambda *a: CodeScanner(
+                    self.on_scan_code,
+                    self.attributes['scan_code'] == 'loop'))
+
     def get_fields(self):
         return list(self.widgets.keys())
 
@@ -512,13 +544,10 @@ class ViewForm(View):
         if record:
             # Force to set fields in record
             # Get first the lazy one from the view to reduce number of requests
-            field_names = set()
-            for name in self.widgets:
-                field = record.group.fields[name]
-                field_names.add(name)
-                field_names.update(f for f in field.attrs.get('depends', [])
-                    if (not f.startswith('_parent')
-                        and f in record.group.fields))
+            field_names = set(self.get_fields())
+            for name, field in record.group.fields.items():
+                if self.view_id in field.views:
+                    field_names.add(name)
             fields = []
             for name in field_names:
                 field = record.group.fields[name]
@@ -589,3 +618,13 @@ class ViewForm(View):
             self.screen.button(widget.attrs)
         finally:
             widget.handler_unblock_by_func(self.button_clicked)
+
+    def on_scan_code(self, code):
+        if self.record:
+            modified = self.record.on_scan_code(
+                code, self.attributes.get('scan_code_depends', []))
+            if self.attributes['scan_code'] == 'submit':
+                window = self.widget.get_toplevel()
+                if window and isinstance(window, Gtk.Window):
+                    GLib.idle_add(lambda: window and window.activate_default())
+            return modified
