@@ -2,7 +2,8 @@
 # this repository contains the full copyright notices and license terms.
 from pathlib import Path
 
-from gi.repository import Gtk, Gdk, GLib
+from gi.repository import Gdk, GLib, Gtk
+
 try:
     from gi.repository import EvinceDocument, EvinceView
     EvinceDocument.init()
@@ -11,8 +12,8 @@ except ImportError:
 
 from tryton.common import data2pixbuf, resize_pixbuf
 
-from .widget import Widget
 from .binary import BinaryMixin
+from .widget import Widget
 
 
 class Document(BinaryMixin, Widget):
