@@ -2,18 +2,19 @@
 # this repository contains the full copyright notices and license terms.
 import gettext
 import webbrowser
-from urllib.parse import urljoin, urlencode
+from urllib.parse import urlencode, urljoin
 
 from gi.repository import Gtk
 
-from .char import Char
-from .widget import Widget, TranslateMixin
 import tryton.common as common
-from tryton.config import CONFIG
 from tryton.common.common import selection
 from tryton.common.underline import set_underline
+from tryton.config import CONFIG
 from tryton.rpc import CONNECTION
 
+from .char import Char
+from .widget import TranslateMixin, Widget
+
 _ = gettext.gettext
 
 
@@ -44,6 +45,7 @@ class URL(Char):
                 value = icon
             self.button.set_image(common.IconFactory.get_image(
                     value, Gtk.IconSize.SMALL_TOOLBAR))
+        self.button.set_sensitive(bool(self.entry.get_text()))
 
     def set_tooltips(self):
         value = self.entry.get_text()
@@ -54,14 +56,6 @@ class URL(Char):
             self.tooltips.set_tip(self.button, '')
             self.tooltips.disable()
 
-    def _readonly_set(self, value):
-        super(URL, self)._readonly_set(value)
-        if value:
-            self.entry.hide()
-        else:
-            self.entry.show()
-        self.button.set_sensitive(True)
-
     def button_clicked(self, widget):
         value = self.entry.get_text()
         if value:
