@@ -5,8 +5,7 @@ import gettext
 from gi.repository import Gdk, GLib, Gtk
 
 import tryton.common as common
-from tryton.common import RPCExecute, RPCException
-from tryton.common import TRYTON_ICON
+from tryton.common import TRYTON_ICON, RPCException, RPCExecute
 from tryton.common.underline import set_underline
 from tryton.common.widget_style import widget_class
 from tryton.gui import Main
@@ -17,7 +16,7 @@ _ = gettext.gettext
 
 class Widget(object):
     expand = False
-    default_width_chars = 25
+    default_width_chars = 8
 
     def __init__(self, view, attrs):
         super(Widget, self).__init__()
@@ -78,8 +77,8 @@ class Widget(object):
         def send(value):
             if not self.widget.props.window:
                 return
-            if self.record and self.get_value() == value:
-                self.record.signal('record-modified')
+            if self.record and self.get_value() == value and self.modified:
+                self.view.screen.record_modified(display=False)
 
         def get_value():
             if not self.widget.props.window:
@@ -126,6 +125,9 @@ class Widget(object):
         self.invisible_set(self.attrs.get(
                 'invisible', states.get('invisible', False)))
 
+    def get_value(self):
+        pass
+
     def set_value(self):
         pass
 
@@ -202,17 +204,15 @@ class TranslateDialog(NoModal):
             widget.set_vexpand(self.widget.expand)
             widget.set_hexpand(True)
             grid.attach(widget, 1, i, 1, 1)
-            editing = Gtk.CheckButton()
+            editing = Gtk.ToggleButton(label=_("Edit"))
             editing.connect('toggled', self.editing_toggled, widget)
             editing.props.sensitive = not readonly
-            tooltips.set_tip(editing, _('Edit'))
             grid.attach(editing, 2, i, 1, 1)
-            fuzzy = Gtk.CheckButton()
-            fuzzy.set_active(value != fuzzy_value)
-            fuzzy.props.sensitive = False
-            tooltips.set_tip(fuzzy, _('Fuzzy'))
-            grid.attach(fuzzy, 4, i, 1, 1)
-            self.widgets[language['code']] = (widget, editing, fuzzy)
+            if value != fuzzy_value:
+                fuzzy = Gtk.Label(_("Fuzzy"))
+                widget_class(fuzzy, 'warning', True)
+                grid.attach(fuzzy, 4, i, 1, 1)
+            self.widgets[language['code']] = (widget, editing)
 
         tooltips.enable()
         vbox = Gtk.VBox()
@@ -239,7 +239,7 @@ class TranslateDialog(NoModal):
     def response(self, win, response):
         if response == Gtk.ResponseType.OK:
             for code, widget in self.widgets.items():
-                widget, editing, fuzzy = widget
+                widget, editing = widget
                 if not editing.get_active():
                     continue
                 value = self.widget.translate_widget_get(widget)
