@@ -138,7 +138,7 @@ class Graph(Gtk.DrawingArea):
     def action_keyword(self, ids):
         if not ids:
             return
-        ctx = self.group._context.copy()
+        ctx = self.group.local_context.copy()
         if 'active_ids' in ctx:
             del ctx['active_ids']
         if 'active_id' in ctx:
