@@ -7,7 +7,6 @@ import sys
 from functools import wraps
 
 from gi.repository import Gdk, GLib, GObject, Gtk
-from pygtkcompat.generictreemodel import GenericTreeModel
 
 import tryton.common as common
 from tryton.common import (
@@ -21,6 +20,7 @@ from tryton.pyson import PYSONDecoder
 
 from . import View, XMLViewParser
 from .list_gtk.editabletree import EditableTreeView, TreeView
+from .list_gtk.generictreemodel import GenericTreeModel
 from .list_gtk.widget import (
     M2M, M2O, O2M, O2O, URL, Affix, Binary, Boolean, Button, Char, Date, Dict,
     Float, Image, Int, MultiSelection, ProgressBar, Reference, Selection, Text,
@@ -1061,6 +1061,9 @@ class ViewTree(View):
                     records.append(model.get_value(iter_, 0))
                 if self.record not in records:
                     self.record = records[0]
+                else:
+                    # Force record_message
+                    self.record = self.record
             else:
                 self.record = None
 
