@@ -1,28 +1,29 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from itertools import chain
 import gettext
+from itertools import chain
 
 from gi.repository import Gdk, Gtk, Pango
 
-from .infobar import InfoBar
 import tryton.common as common
-from tryton.common import TRYTON_ICON, MODELNAME
+from tryton.common import MODELNAME, TRYTON_ICON
 from tryton.common.domain_parser import quote
 from tryton.common.underline import set_underline
 from tryton.common.widget_style import widget_class
 from tryton.gui import Main
 from tryton.gui.window.nomodal import NoModal
 
+from .infobar import InfoBar
+
 _ = gettext.gettext
 
 
 class WinForm(NoModal, InfoBar):
     "Form window"
 
-    def __init__(self, screen, callback, view_type='form',
+    def __init__(self, screen, callback=None, view_type='form',
             new=False, many=0, domain=None, context=None,
-            save_current=False, title='', rec_name=None):
+            save_current=False, title='', defaults=None):
         tooltips = common.Tooltips()
         NoModal.__init__(self)
         self.screen = screen
@@ -51,7 +52,7 @@ class WinForm(NoModal, InfoBar):
             self.destroy()
             return
         if new:
-            self.screen.new(rec_name=rec_name)
+            self.screen.new(defaults=defaults)
         self.win = Gtk.Dialog(
             title=_('Link'), transient_for=self.parent,
             destroy_with_parent=True)
@@ -74,11 +75,12 @@ class WinForm(NoModal, InfoBar):
         self.but_new = None
 
         self._initial_value = None
-        if view_type == 'form':
+        self.view_type = view_type
+        if view_type == 'form' and not readonly:
             if new:
                 label, icon = _("Delete"), 'tryton-delete'
             else:
-                label, icon = _("Cancel"), 'tryton-cancel'
+                label, icon = _("Discard changes"), 'tryton-cancel'
                 record = self.screen.current_record
                 self._initial_value = record.get_on_change_value()
                 if record.parent and record.parent_name in record.group.fields:
@@ -92,15 +94,20 @@ class WinForm(NoModal, InfoBar):
             self.but_cancel.set_always_show_image(True)
 
         if new and self.many:
+            if self.save_current and not readonly:
+                label = _("Save and New")
+            else:
+                label = _("Add and New")
             self.but_new = self.win.add_button(
-                set_underline(_("New")), Gtk.ResponseType.ACCEPT)
+                set_underline(label), Gtk.ResponseType.ACCEPT)
             self.but_new.set_image(common.IconFactory.get_image(
                     'tryton-create', Gtk.IconSize.BUTTON))
             self.but_new.set_always_show_image(True)
             self.but_new.set_accel_path('<tryton>/Form/New', self.accel_group)
 
         if self.save_current and not readonly:
-            self.but_ok = Gtk.Button(label=_('_Save'), use_underline=True)
+            self.but_ok = Gtk.Button(
+                label=set_underline(_("Save")), use_underline=True)
             self.but_ok.set_image(common.IconFactory.get_image(
                     'tryton-save', Gtk.IconSize.BUTTON))
             self.but_ok.set_always_show_image(True)
@@ -108,11 +115,15 @@ class WinForm(NoModal, InfoBar):
             self.but_ok.set_can_default(True)
             self.but_ok.show()
             self.win.add_action_widget(self.but_ok, Gtk.ResponseType.OK)
-            if not new:
-                self.but_ok.props.sensitive = False
         else:
+            if readonly or view_type == 'tree':
+                label = _("Close")
+            elif new:
+                label = _("Add")
+            else:
+                label = _("Apply changes")
             self.but_ok = self.win.add_button(
-                set_underline(_("OK")), Gtk.ResponseType.OK)
+                set_underline(label), Gtk.ResponseType.OK)
             self.but_ok.set_image(common.IconFactory.get_image(
                     'tryton-ok', Gtk.IconSize.BUTTON))
             self.but_ok.set_always_show_image(True)
@@ -272,21 +283,14 @@ class WinForm(NoModal, InfoBar):
 
         scroll.add(self.screen.screen_container.alternate_viewport)
 
-        self.create_info_bar()
         self.win.vbox.pack_start(
-            self.info_bar, expand=False, fill=True, padding=0)
+            self.create_info_bar(), expand=False, fill=True, padding=0)
 
+        self.screen.windows.append(self)
         if view_type == 'tree':
-            self.screen.signal_connect(self, 'record-message', self._sig_label)
             self.screen.screen_container.alternate_viewport.connect(
                     'key-press-event', self.on_keypress)
 
-        if self.save_current and not new:
-            self.screen.signal_connect(self, 'record-message',
-                self.activate_save)
-            self.screen.signal_connect(self, 'record-modified',
-                self.activate_save)
-
         self.register()
         self.show()
 
@@ -340,7 +344,6 @@ class WinForm(NoModal, InfoBar):
             if result:
                 ids = [x[0] for x in result]
                 self.screen.load(ids, modified=True)
-                self.screen.display(res_id=ids[0])
             self.screen.set_cursor()
             self.wid_text.set_text('')
 
@@ -349,22 +352,24 @@ class WinForm(NoModal, InfoBar):
         win.screen.search_filter(quote(value))
         win.show()
 
-    def _sig_label(self, screen, signal_data):
+    def record_message(self, position, size, *args):
+        if self.view_type != 'tree':
+            return
         name = '_'
-        access = common.MODELACCESS[screen.model_name]
+        access = common.MODELACCESS[self.screen.model_name]
         deletable = True
-        if screen.current_record:
-            deletable = screen.current_record.deletable
+        if self.screen.current_record:
+            deletable = self.screen.current_record.deletable
         readonly = self.screen.group.readonly
-        if signal_data[0] >= 1:
-            name = str(signal_data[0])
+        if position >= 1:
+            name = str(position)
             if self.domain is not None:
                 self.but_remove.set_sensitive(True)
-            if signal_data[0] < signal_data[1]:
+            if position < size:
                 self.but_next.set_sensitive(True)
             else:
                 self.but_next.set_sensitive(False)
-            if signal_data[0] > 1:
+            if position > 1:
                 self.but_pre.set_sensitive(True)
             else:
                 self.but_pre.set_sensitive(False)
@@ -380,16 +385,11 @@ class WinForm(NoModal, InfoBar):
             self.but_pre.set_sensitive(False)
             if self.domain is not None:
                 self.but_remove.set_sensitive(False)
-        line = '(%s/%s)' % (name, signal_data[1])
+        line = '(%s/%s)' % (name, position)
         self.label.set_text(line)
 
-    def activate_save(self, *args):
-        modified = self.screen.modified()
-        # Keep sensible as change could have been trigger by a Many2One edition
-        sensitive = modified or self.but_ok.props.sensitive
-        self.but_ok.props.sensitive = sensitive
-        self.win.set_default_response(
-            Gtk.ResponseType.OK if sensitive else Gtk.ResponseType.CANCEL)
+    def record_modified(self, *args):
+        self.info_bar_refresh()
 
     def close(self, widget):
         widget.stop_emission_by_name('close')
@@ -422,12 +422,12 @@ class WinForm(NoModal, InfoBar):
                             if record:
                                 validate = record.pre_validate()
             if not validate:
-                self.message_info(self.screen.invalid_message(),
-                    Gtk.MessageType.ERROR)
+                self.info_bar_add(
+                    self.screen.invalid_message(), Gtk.MessageType.ERROR)
                 self.screen.set_cursor()
                 self.screen.display()
                 return
-            self.message_info()
+            self.info_bar_clear()
             if response_id == Gtk.ResponseType.ACCEPT:
                 self.new()
                 return
@@ -442,13 +442,14 @@ class WinForm(NoModal, InfoBar):
             elif record.modified:
                 record.cancel()
                 record.reload()
-                record.signal('record-changed')
+                record.set_modified()
             if added:
                 record.modified_fields.setdefault('id')
             result = False
         else:
             result = (response_id not in cancel_responses) and not readonly
-        self.callback(result)
+        if self.callback:
+            self.callback(result)
         self.destroy()
 
     def new(self):
@@ -462,12 +463,15 @@ class WinForm(NoModal, InfoBar):
             self.win.set_default_response(Gtk.ResponseType.OK)
 
     def destroy(self):
+        try:
+            self.screen.windows.remove(self)
+        except ValueError:
+            pass
         self.screen.screen_container.alternate_view = False
         viewport = self.screen.screen_container.alternate_viewport
         if viewport and viewport.get_parent():
             viewport.get_parent().remove(viewport)
         self.screen.switch_view(view_type=self.prev_view.view_type)
-        self.screen.signal_unconnect(self)
         if getattr(self, 'win', None):
             self.win.destroy()
         NoModal.destroy(self)
