@@ -1,6 +1,8 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
+import base64
 import csv
+import datetime as dt
 import gettext
 import locale
 from decimal import Decimal
@@ -8,8 +10,7 @@ from decimal import Decimal
 from gi.repository import Gtk
 
 import tryton.common as common
-from tryton.common import RPCExecute, RPCException
-from tryton.common.datetime_ import date_parse
+from tryton.common import RPCException, RPCExecute
 from tryton.gui.window.win_csv import WinCSV
 
 _ = gettext.gettext
@@ -116,8 +117,8 @@ class WinImport(WinCSV):
                 open(fname, 'r', encoding=encoding, newline=''),
                 quotechar=self.get_quotechar(),
                 delimiter=self.get_delimiter())
-        except IOError:
-            common.warning(_('Error opening CSV file'), _('Error'))
+        except (IOError, UnicodeDecodeError, csv.Error) as exception:
+            common.warning(str(exception), _("Detection failed"))
             return True
         self.sig_unsel_all()
         word = ''
@@ -146,8 +147,9 @@ class WinImport(WinCSV):
                     name = self.fields[word][0]
                     field = word
                 else:
-                    common.warning(_('Error processing the file at field %s.')
-                        % word, _('Error'))
+                    common.warning(
+                        _('Unknown column header "%s"') % word,
+                        _('Error'))
                     return True
                 num = self.model2.append()
                 self.model2.set(num, 0, name, 1, field)
@@ -191,28 +193,39 @@ class WinImport(WinCSV):
         skip = self.csv_skip.get_value_as_int()
         encoding = self.get_encoding()
         locale_format = self.csv_locale.get_active()
-        reader = csv.reader(
-            open(fname, 'r', encoding=encoding),
-            quotechar=self.get_quotechar(),
-            delimiter=self.get_delimiter())
-        data = []
-        for i, line in enumerate(reader):
-            if i < skip or not line:
-                continue
-            row = []
-            for field, val in zip(fields, line):
-                if locale_format and val:
-                    type_ = self.fields_data[field]['type']
-                    if type_ in ['integer', 'biginteger']:
-                        val = locale.atoi(val)
-                    elif type_ == 'float':
-                        val = locale.atof(val)
-                    elif type_ == 'numeric':
-                        val = Decimal(locale.delocalize(val))
-                    elif type_ in ['date', 'datetime']:
-                        val = date_parse(val, common.date_format())
-                row.append(val)
-            data.append(row)
+        try:
+            reader = csv.reader(
+                open(fname, 'r', encoding=encoding),
+                quotechar=self.get_quotechar(),
+                delimiter=self.get_delimiter())
+            data = []
+            for i, line in enumerate(reader):
+                if i < skip or not line:
+                    continue
+                row = []
+                for field, val in zip(fields, line):
+                    if locale_format and val:
+                        type_ = self.fields_data[field]['type']
+                        if type_ == 'integer':
+                            val = locale.atoi(val)
+                        elif type_ == 'float':
+                            val = locale.atof(val)
+                        elif type_ == 'numeric':
+                            val = Decimal(locale.delocalize(val))
+                        elif type_ == 'date':
+                            val = dt.datetime.strptime(
+                                val, common.date_format()).date()
+                        elif type_ == 'datetime':
+                            val = dt.datetime.strptime(
+                                val, common.date_format() + ' %X')
+                        elif type_ == 'binary':
+                            val = base64.b64decode(val)
+                    row.append(val)
+                data.append(row)
+        except (IOError, UnicodeDecodeError, csv.Error, ValueError) \
+                as exception:
+            common.warning(str(exception), _("Import failed"))
+            return
         try:
             count = RPCExecute(
                 'model', self.model, 'import_data', fields, data,
