@@ -1,20 +1,20 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import logging
 import gettext
+import logging
 
 from gi.repository import Gdk, Gtk, Pango
 
-from tryton.signal_event import SignalEvent
 import tryton.common as common
-from tryton.gui.window.view_form.screen import Screen
-from tryton.gui import Main
-from tryton.exceptions import TrytonServerError
-from tryton.gui.window.nomodal import NoModal
+from tryton.common import (
+    TRYTON_ICON, RPCContextReload, RPCException, RPCExecute)
 from tryton.common.button import Button
-from tryton.common import RPCExecute, RPCException, RPCContextReload
-from tryton.common import TRYTON_ICON
 from tryton.common.widget_style import widget_class
+from tryton.exceptions import TrytonServerError
+from tryton.gui import Main
+from tryton.gui.window.nomodal import NoModal
+from tryton.gui.window.view_form.screen import Screen
+
 from .infobar import InfoBar
 from .tabcontent import TabContent
 
@@ -107,8 +107,11 @@ class Wizard(InfoBar):
                 self.update(view['fields_view'], view['buttons'])
 
                 self.screen.new(default=False)
-                self.screen.current_record.set_default(view['defaults'])
-                self.update_buttons(self.screen.current_record)
+                if 'defaults' in view:
+                    self.screen.current_record.set_default(view['defaults'])
+                if 'values' in view:
+                    self.screen.current_record.set(view['values'])
+                self.update_buttons()
                 self.screen.set_cursor()
 
                 self.screen_state = view['state']
@@ -155,8 +158,8 @@ class Wizard(InfoBar):
                 process_exception=False, callback=end_callback)
         except Exception:
             logger.warn(
-                _("Unable to delete wizard %s") % self.session_id,
-                exc_info=True)
+                "Unable to delete session %s of wizard %s",
+                self.session_id, self.action, exc_info=True)
 
     def clean(self):
         for widget in self.widget.get_children():
@@ -171,10 +174,10 @@ class Wizard(InfoBar):
         if (button_attrs.get('validate', True)
                 and not self.screen.current_record.validate()):
             self.screen.display(set_cursor=True)
-            self.message_info(
+            self.info_bar_add(
                 self.screen.invalid_message(), Gtk.MessageType.ERROR)
             return
-        self.message_info()
+        self.info_bar_clear()
         self.state = state
         self.process()
 
@@ -186,10 +189,12 @@ class Wizard(InfoBar):
         button.show()
         return button
 
-    def _record_changed(self, screen, record):
-        self.update_buttons(record)
+    def record_modified(self):
+        self.update_buttons()
+        self.info_bar_refresh()
 
-    def update_buttons(self, record):
+    def update_buttons(self):
+        record = self.screen.current_record
         for button in self.states.values():
             button.state_set(record)
 
@@ -198,12 +203,14 @@ class Wizard(InfoBar):
         for button in buttons:
             self._get_button(button)
 
+        if self.screen:
+            self.screen.windows.remove(self)
+
         self.screen = Screen(view['model'], mode=[], context=self.context)
         self.screen.add_view(view)
         self.screen.switch_view()
         self.screen.widget.show()
-        self.screen.signal_connect(self, 'group-changed',
-            self._record_changed)
+        self.screen.windows.append(self)
 
         title = Gtk.Label(
             label=common.ellipsize(self.name, 80),
@@ -239,12 +246,11 @@ class Wizard(InfoBar):
         self.widget.pack_start(
             self.scrolledwindow, expand=True, fill=True, padding=0)
 
-        self.create_info_bar()
         self.widget.pack_start(
-            self.info_bar, expand=False, fill=True, padding=0)
+            self.create_info_bar(), expand=False, fill=True, padding=0)
 
 
-class WizardForm(Wizard, TabContent, SignalEvent):
+class WizardForm(Wizard, TabContent):
     "Wizard"
 
     def __init__(self, name=''):
