@@ -2,8 +2,9 @@
 # this repository contains the full copyright notices and license terms.
 import http.client
 import logging
-import socket
 import os
+import socket
+
 try:
     from http import HTTPStatus
 except ImportError:
@@ -12,11 +13,11 @@ except ImportError:
 from functools import partial
 
 from tryton import bus, device_cookie, fingerprints
-from tryton.jsonrpc import ServerProxy, ServerPool, Fault
-from tryton.config import get_config_dir
+from tryton.config import CONFIG, get_config_dir
 from tryton.exceptions import TrytonServerError, TrytonServerUnavailable
-from tryton.config import CONFIG
+from tryton.jsonrpc import Fault, ServerPool, ServerProxy
 
+logger = logging.getLogger(__name__)
 CONNECTION = None
 _USER = None
 CONTEXT = {}
@@ -44,12 +45,12 @@ context_reset()
 def db_list(host, port):
     try:
         connection = ServerProxy(host, port)
-        logging.getLogger(__name__).info('common.db.list()')
+        logger.info('common.db.list()')
         result = connection.common.db.list()
-        logging.getLogger(__name__).debug(repr(result))
+        logger.debug('%r', result)
         return result
     except Fault as exception:
-        logging.getLogger(__name__).debug(exception.faultCode)
+        logger.debug(exception.faultCode)
         if exception.faultCode == str(HTTPStatus.FORBIDDEN.value):
             return []
         else:
@@ -59,16 +60,53 @@ def db_list(host, port):
 def server_version(host, port):
     try:
         connection = ServerProxy(host, port)
-        logging.getLogger(__name__).info(
-            'common.server.version(None, None)')
+        logger.info('common.server.version(None, None)')
         result = connection.common.server.version()
-        logging.getLogger(__name__).debug(repr(result))
+        logger.debug('%r', result)
         return result
     except Exception as e:
-        logging.getLogger(__name__).error(e)
+        logger.exception(e)
         return None
 
 
+def authentication_services(host, port):
+    try:
+        connection = ServerProxy(host, port)
+        logger.info('common.authentication.services()')
+        services = connection.common.authentication.services()
+        logger.debug('%r', services)
+        return connection.url, services
+    except Exception as e:
+        logger.exception(e)
+        return '', []
+
+
+def set_service_session(parameters):
+    from tryton import common
+    global CONNECTION, _USER
+    host = CONFIG['login.host']
+    hostname = common.get_hostname(host)
+    port = common.get_port(host)
+    database = CONFIG['login.db']
+    CONFIG['login.login'] = username = parameters.get('login', [''])[0]
+    try:
+        user_id = int(parameters.get('user_id', [None])[0])
+    except TypeError:
+        pass
+    session = parameters.get('session', [''])[0]
+    if 'renew' in parameters:
+        renew_id = int(parameters.get('renew', [-1])[0])
+        if _USER != renew_id:
+            raise ValueError
+    _USER = user_id
+    session = ':'.join(map(str, [username, user_id, session]))
+    if CONNECTION is not None:
+        CONNECTION.close()
+    CONNECTION = ServerPool(
+        hostname, port, database, session=session, cache=not CONFIG['dev'])
+    bus.listen(CONNECTION)
+
+
 def login(parameters):
     from tryton import common
     global CONNECTION, _USER
@@ -80,10 +118,9 @@ def login(parameters):
     language = CONFIG['client.lang']
     parameters['device_cookie'] = device_cookie.get()
     connection = ServerProxy(hostname, port, database)
-    logging.getLogger(__name__).info('common.db.login(%s, %s, %s)'
-        % (username, 'x' * 10, language))
+    logger.info('common.db.login(%s, %s, %s)', username, 'x' * 10, language)
     result = connection.common.db.login(username, parameters, language)
-    logging.getLogger(__name__).debug(repr(result))
+    logger.debug('%r', result)
     _USER = result[0]
     session = ':'.join(map(str, [username] + result))
     if CONNECTION is not None:
@@ -98,7 +135,7 @@ def logout():
     global CONNECTION, _USER
     if CONNECTION is not None:
         try:
-            logging.getLogger(__name__).info('common.db.logout()')
+            logger.info('common.db.logout()')
             with CONNECTION() as conn:
                 conn.common.db.logout()
         except (Fault, socket.error, http.client.CannotSendRequest):
@@ -115,12 +152,12 @@ def execute(*args):
     try:
         name = '.'.join(args[:3])
         args = args[3:]
-        logging.getLogger(__name__).info('%s%s' % (name, args))
+        logger.info('%s%r', name, args)
         with CONNECTION() as conn:
             result = getattr(conn, name)(*args)
     except (http.client.CannotSendRequest, socket.error) as exception:
         raise TrytonServerUnavailable(*exception.args)
-    logging.getLogger(__name__).debug(repr(result))
+    logger.debug('%r', result)
     return result
 
 
