@@ -768,6 +768,9 @@ class DomainParserTestCase(TestCase):
         self.assertEqual(
             dom.string([('reference.rec_name', 'ilike', '%bar%', 'spam')]),
             'Reference: Spam,bar')
+        self.assertEqual(
+            dom.string([('reference.rec_name', '=', None, 'spam')]),
+            "Reference: =Spam,")
         self.assertEqual(
             dom.string([('reference', 'in', ['foo', 'bar'])]),
             'Reference: foo;bar')
@@ -1081,6 +1084,10 @@ class DomainParserTestCase(TestCase):
             rlist(dom.parse_clause([('Reference', None, 'Spam,bar')])), [
                 ('reference.rec_name', 'ilike', '%bar%', 'spam'),
                 ])
+        self.assertEqual(
+            rlist(dom.parse_clause([('Reference', None, 'Spam,')])), [
+                ('reference.rec_name', 'ilike', '%', 'spam'),
+                ])
         self.assertEqual(
             rlist(dom.parse_clause([('Reference', None, ['foo', 'bar'])])), [
                 ('reference', 'in', ['foo', 'bar']),
