@@ -1,13 +1,14 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 "Translate"
-import os
-import locale
 import gettext
+import locale
 import logging
-import gtk
+import os
 import sys
 
+from gi.repository import Gtk
+
 from tryton.config import CURRENT_DIR
 
 _ = gettext.gettext
@@ -85,10 +86,8 @@ _LOCALE2WIN32 = {
     'ga': 'Scottish Gaelic',
     'gl_ES': 'Galician_Spain',
     'gu': 'Gujarati_India',
-    'he_IL': 'Hebrew',
     'he_IL': 'Hebrew_Israel',
     'hi_IN': 'Hindi',
-    'hi_IN': 'Hindi',
     'hr_HR': 'Croatian',
     'hu_HU': 'Hungarian',
     'hu': 'Hungarian_Hungary',
@@ -155,10 +154,15 @@ def setlang(lang=None, locale_dict=None):
     "Set language"
     locale_dir = os.path.join(CURRENT_DIR, 'data/locale')
     if not os.path.isdir(locale_dir):
-        # do not import when frozen
-        import pkg_resources
-        locale_dir = pkg_resources.resource_filename(
-            'tryton', 'data/locale')
+        try:
+            import importlib.resources
+            ref = importlib.resources.files('tryton') / 'data/locale'
+            with importlib.resources.as_file(ref) as path:
+                locale_dir = path
+        except ImportError:
+            import pkg_resources
+            locale_dir = pkg_resources.resource_filename(
+                'tryton', 'data/locale')
     if lang:
         encoding = locale.getdefaultlocale()[1]
         if not encoding:
@@ -190,7 +194,7 @@ def setlang(lang=None, locale_dict=None):
 
     if locale_dict:
         conv = locale.localeconv()
-        for field in locale_dict.keys():
+        for field in list(locale_dict.keys()):
             if field == 'date':
                 continue
             conv[field] = locale_dict[field]
@@ -199,6 +203,7 @@ def setlang(lang=None, locale_dict=None):
 
 def set_language_direction(direction):
     if direction == 'rtl':
-        gtk.widget_set_default_direction(gtk.TEXT_DIR_RTL)
+        direction = Gtk.TextDirection.RTL
     else:
-        gtk.widget_set_default_direction(gtk.TEXT_DIR_LTR)
+        direction = Gtk.TextDirection.LTR
+    Gtk.Widget.set_default_direction(direction)
